package gonganxitong

import (
	"time"

	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/aaa"
	"shylinux.com/x/icebergs/base/nfs"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/gonganxitong/model"
)

type user struct {
	Table
	setting    setting
	community  string `data:"20240724-community"`
	template   string `data:"4b-Z_r8dZmm1pHdd2h4A10VVYX4OIHvemlLjsHKBj2s"`
	create     string `name:"create open_id* usernick avatar"`
	modify     string `name:"modify user_name user_info" role:"void"`
	email      string `name:"email email*"`
	list       string `name:"list uid auto" role:"void"`
	authCreate string `name:"authCreate city_name* company_name* auth_name* auth_info" role:"void"`
}

func (s user) Create(m *ice.Message, arg ...string) {
	if m.IsTech() {
		m.OptionDefault(aaa.AVATAR, m.Option(ice.MSG_AVATAR))
		m.OptionDefault(aaa.USERNICK, m.Option(ice.MSG_USERNICK))
	}
	if s.Table.Select(m, m.OptionSimple(model.OPEN_ID)...).Length() == 0 {
		s.Insert(m, kit.Simple(m.OptionSimple(model.OPEN_ID, model.AVATAR), model.NAME, m.Option(aaa.USERNICK))...)
		m.Option(model.USER_UID, m.Result())
	} else {
		s.Update(m, kit.Dict(m.OptionSimple(model.AVATAR), model.NAME, m.Option(aaa.USERNICK)), m.AppendSimple(model.UID)...)
		m.Option(model.USER_UID, m.Append(model.UID))
	}
}
func (s user) Modify(m *ice.Message, arg ...string) {
	s.Update(m, kit.Dict(
		model.NAME, m.Option(model.USER_NAME),
		model.INFO, m.Option(model.USER_INFO),
	), model.UID, m.Option(model.USER_UID))
}
func (s user) Delete(m *ice.Message, arg ...string) {
	m.Options(model.NAME, "profile", model.VALUE, "off")
	m.Cmd(s.setting, s.Update)
}
func (s user) AuthCreate(m *ice.Message, arg ...string) {
	s.Table.AuthCreate(m, m.Option(model.CITY_NAME), m.Option(model.COMPANY_NAME), 2, m.Option(model.AUTH_NAME), m.Option(model.AUTH_INFO), model.UID, m.Option(model.USER_UID))
}
func (s user) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		if m.IsTech() {
			s.Select(m, arg...).Table(func(value ice.Maps) {
				if value[model.UID] != m.Option(model.USER_UID) {
					m.PushButton(s.SetCookie)
				} else {
					m.PushButton()
				}
			})
		}
	} else {
		s.Select(m, model.UID, arg[0])
		if m.Append(model.AUTH_UID) == "" && arg[0] == m.Option(model.USER_UID) {
			m.EchoInfoButton("请申请个人认证", s.AuthCreate)
		}
	}
}
func (s user) SetCookie(m *ice.Message, arg ...string) {
	m.ProcessCookie(model.USER_UID, m.Option(model.UID))
}
func (s user) Email(m *ice.Message, arg ...string) {
	s.Table.Update(m, kit.Dict(m.OptionSimple(model.EMAIL)), model.UID, m.Option(model.USER_UID))
}
func (s user) SendTemplate(m *ice.Message, arg ...string) { // from uid url type name hash
	msg := s.Select(m, model.UID, kit.Select(m.Option(model.USER_UID), arg, 1))
	m.Cmdy("web.chat.wx.template", "", m.Config(nfs.TEMPLATE), msg.Append(model.OPEN_ID), kit.Select("", arg, 2),
		"thing7", kit.Select("", arg, 3), "thing12", kit.Select("", arg, 4), "character_string2", kit.Cut(kit.Select("", arg, 5), 6),
		"time11", time.Now().Format("2006年01月02日 15:04"), "thing18", kit.Select(kit.Select(m.Option(ice.MSG_USERNAME), m.Option(ice.MSG_USERNICK)), arg, 0),
	)
}

func init() { ice.TeamCtxCmd(user{}) }
